﻿//     ___                        _                    _     
//    / _ \_   _ _ __  _ __   ___| |_  /\/\   __ _ ___| |_ ___ _ __ 
//   / /_)/ | | | '_ \| '_ \ / _ \ __|/    \ / _` / __| __/ _ \ '__|
//  / ___/| |_| | |_) | |_) |  __/ |_/ /\/\ \ (_| \__ \ ||  __/ |  
//  \/     \__,_| .__/| .__/ \___|\__\/    \/\__,_|___/\__\___|_|  
//              |_|   |_|                   
//                                           
using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;
using Newtonsoft.Json;
using System.Runtime.Serialization.Formatters;

namespace PuppetMaster
{
    public class CustomPoses
    {
        public static Dictionary<string, PoseSet> Poses = new Dictionary<string, PoseSet>();

        public class PoseSet
        {
            public Dictionary<string, string> LimbAngles;

            public float AnimationSpeedMultiplier        = 1f;
            public float DragInfluence                   = 1.0f;
            public float RandomInfluence                 = 0f;
            public float RandomSpeed                     = 0f;
            public float Rigidity                        = 5f;
            public float TimeOffset                      = 0f;
            public float UprightForceMultiplier          = 1.0f;

            public bool Animated = false;
            public bool ShouldStumble                    = true;
            public bool ShouldStandUpright               = true;

            public PoseState State                       = 0;
        }


        public static Dictionary<string, int> Assign(PersonBehaviour Peep)
        {
            Dictionary<string, int> poseList = new Dictionary<string, int>();

            foreach (string poseName in Poses.Keys)
            {
                poseList.Add(poseName, GetCustomPose(Peep, poseName));
            }

            return poseList;
        }

        // - - - - - - - - - - - - - - - - - - -
        //
        //  FN: GET CUSTOM POSE
        //  Checks if peep has a custom pose and creates it if necessary
        //
        public static int GetCustomPose(PersonBehaviour Peep, string PoseName)
        {
            if (!Poses.ContainsKey(PoseName))
            {
                Util.Notify( "<color=red>Missing Custom Pose:</color> " + PoseName, Util.VerboseLevels.Full);
                return 0;
            }
            
            for (int pnum = Peep.Poses.Count; --pnum >= 0;)
                if (Peep.Poses[pnum].Name == PoseName) return pnum;

            RagdollPose customPose = new RagdollPose()
            {
                Name                     = PoseName,
                Angles                   = new List<RagdollPose.LimbPose>(),
                ShouldStandUpright       = Poses[PoseName].ShouldStandUpright,
                AnimationSpeedMultiplier = Poses[PoseName].AnimationSpeedMultiplier,
                UprightForceMultiplier   = Poses[PoseName].UprightForceMultiplier,
                DragInfluence            = Poses[PoseName].DragInfluence,
                Rigidity                 = Poses[PoseName].Rigidity,
                State                    = PoseState.Rest,

            };

            string limbAngleIn;

            foreach (LimbBehaviour limb in Peep.Limbs)
            {
                if (Poses[PoseName].LimbAngles.TryGetValue(limb.name, out limbAngleIn) )
                {
                    float startAngle = 0.0f;
                    float endAngle   = 0.0f;

                    if (limbAngleIn.Contains(":"))
                    {
                        // Angle range
                        string[] angleRange = limbAngleIn.Split(':');
                        float.TryParse(angleRange[0], out startAngle);
                        float.TryParse(angleRange[1], out endAngle);
                    }
                    else
                    {
                        // single angle
                        float.TryParse(limbAngleIn, out startAngle);

                        endAngle = startAngle;
                    }

                    customPose.Angles.Add(new RagdollPose.LimbPose(limb, startAngle)
                    {
                        Animated   = Poses[PoseName].Animated,
                        StartAngle = startAngle,
                        EndAngle   = endAngle + 5f
                    });
                }
            }

            customPose.ConstructDictionary();

            Peep.Poses.Add(customPose);

            int poseId = Peep.Poses.Count - 1;
            
            Peep.SetPose(Peep.Poses[poseId]);
            
            return poseId;
        }



        // - - - - - - - - - - - - - - - - - - -
        //
        //  FN: CHANGE POSE
        //
        public static void Pose(string poseName, PersonBehaviour Peep)
        {
            int poseId = GetCustomPose(Peep, poseName);

            Peep.OverridePoseIndex = poseId;
        }

        public static void InitCustomPoses()
        {
            string JTCustomPoses = @"
            {
                'JT_Duck': {
                    'LimbAngles':  {
                        'Foot':                 '-24.21083', 
                        'UpperLeg':             '-0.03494752', 
                        'LowerLeg':             '106.0683', 
                        'LowerLegFront':        '78.07973', 
                        'UpperLegFront':        '-75.54669:', 
                        'FootFront':            '0.4032795', 
                        'Head':                 '-0.0002471675', 
                        'LowerBody':            '-0.0002010637',
                        'UpperArm':             '0.1',
                        'UpperArmFront':        '0.1'
                    }, 
                    'ShouldStandUpright': true, 
                    'Animated': false, 
                    'AnimationSpeedMultiplier': 1.0
                },

                'JT_FrontFlip': {
                    'LimbAngles': {
                        'LowerArm':             '-95.09151', 
                        'LowerArmFront':        '-85.78412', 
                        'UpperArm':             '-376.2525', 
                        'Head':                 '6.57841', 
                        'MiddleBody':           '0', 
                        'UpperArmFront':        '-369.4041', 
                        'UpperBody':            '0.4076479', 
                        'LowerBody':            '-29.47088', 
                        'UpperLegFront':        '-111.5341', 
                        'UpperLeg':             '-112.034', 
                        'LowerLeg':             '133.9088', 
                        'LowerLegFront':        '134.5014', 
                        'Foot':                 '-14.32194', 
                        'FootFront':            '-14.81616'
                    },
                    'ShouldStandUpright': true, 
                    'AnimationSpeedMultiplier': 0.5,
                    'Animated': false,
                    'UprightForceMultiplier': 2,
                    'Rigidity': 1.3
                },
                
                
                'JT_Dive_1': {
                    'LimbAngles': {
                        'MiddleBody':           '0', 
                        'UpperArm':             '-22.87857', 
                        'LowerBody':            '-11.74754', 
                        'UpperArmFront':        '-29.61433', 
                        'UpperBody':            '-8.516563', 
                        'LowerArm':             '-34.3288', 
                        'UpperLeg':             '-74.00098', 
                        'UpperLegFront':        '-67.9553', 
                        'LowerArmFront':        '-33.82881', 
                        'Head':                 '9.916041', 
                        'LowerLegFront':        '63.65865', 
                        'FootFront':            '-5.71277', 
                        'LowerLeg':             '60.26822', 
                        'Foot':                 '2.882859'
                    },
                    'ShouldStandUpright': false, 
                    'AnimationSpeedMultiplier': 10.5,
                    'Animated': false,
                    'UprightForceMultiplier': 4.1,
                    'Rigidity': 4.15
                },
               
                'JT_Dive_2': {
                    'LimbAngles': {
                        'MiddleBody':           '0', 
                        'UpperArmFront':        '-149.6484', 
                        'UpperBody':            '-13.5377', 
                        'Head':                 '-3.796226', 
                        'LowerArmFront':        '-9.33497', 
                        'LowerArm':             '0.02141264', 
                        'UpperArm':             '-142.3777', 
                        'LowerLegFront':        '54.19135', 
                        'LowerLeg':             '45.83427', 
                        'UpperLegFront':        '-50.31013', 
                        'UpperLeg':             '-43.46162', 
                        'FootFront':            '-31.9845', 
                        'Foot':                 '-32.13535', 
                        'LowerBody':            '-1.615463'
                    },
                    'ShouldStandUpright': false, 
                    'AnimationSpeedMultiplier': 10.5,
                    'Animated': false,
                    'UprightForceMultiplier': 5,
                    'Rigidity': 1.5
                },

                'JT_Prone': {
                    'LimbAngles': {
                        'Head': '-6.500257', 
                        'LowerBody':            '17.68937', 
                        'MiddleBody':           '0', 
                        'UpperArmFront':        '-18.54307', 
                        'UpperArm':             '-16.35537', 
                        'LowerLegFront':        '9.899184', 
                        'LowerLeg':             '11.33566', 
                        'Foot':                 '0.02769642', 
                        'FootFront':            '0.04737419', 
                        'UpperLegFront':        '16.45826', 
                        'UpperLeg':             '15.87339', 
                        'LowerArmFront':        '-103.8956', 
                        'UpperBody':            '-17.44694', 
                        'LowerArm':             '-102.0114'
                    },
                    'ShouldStandUpright': false, 
                    'AnimationSpeedMultiplier': 10.5,
                    'Animated': false,
                    'UprightForceMultiplier': 1,
                    'Rigidity': 2.2,
                    'State': 8
                },

                'JT_Waving': {
                    'LimbAngles': {
                        'LowerArm':      '-0.08787935', 
                        'LowerBody':     '0.1363826', 
                        'MiddleBody':    '0', 
                        'UpperLegFront': '-0.2402655', 
                        'UpperLeg':      '0.6425738', 
                        'LowerLegFront': '-0.1414893', 
                        'LowerLeg':      '0.06455169', 
                        'UpperArm':      '0.1430857', 
                        'Head':          '0.01760935', 
                        'UpperBody':     '0.01076555', 
                        'LowerArmFront': '-64.94197', 
                        'UpperArmFront': '-87.36889', 
                        'Foot':          '-0.5595296', 
                        'FootFront':     '0.1156012'
                    },
                    'ShouldStandUpright': true, 
                    'Animated': false,
                    'UprightForceMultiplier': 0.1,
                    'Rigidity': 5.2
                },

                'JT_Throw_1': {
                    'LimbAngles': {
                        'LowerArm':      '0.1230614', 
                        'UpperArm':      '-0.117004', 
                        'LowerLeg':      '-0.001100474', 
                        'LowerLegFront': '0.001414623', 
                        'UpperArmFront': '174.287', 
                        'LowerArmFront': '-86.93764', 
                        'Foot':          '0.1132696', 
                        'FootFront':     '0.1011531', 
                        'UpperLeg':      '-0.007332622', 
                        'UpperLegFront': '0.00374265', 
                        'MiddleBody':    '0', 
                        'UpperBody':     '-0.0001217294', 
                        'Head':          '0.0004742446', 
                        'LowerBody':     '-0.000948049'
                    },
                    'ShouldStandUpright': true, 
                    'Animated': false,
                    'UprightForceMultiplier': 0.1,
                    'Rigidity': 5.2
                },

                'JT_Throw_x': {
                    'LimbAngles': {
                        'UpperArmFront': '53.87996', 
                        'UpperArm':      '0.1090901', 
                        'UpperBody':     '-0.003569094', 
                        'Head':          '-0.07377298', 
                        'LowerLegFront': '-0.01318056', 
                        'FootFront':     '-0.8645177', 
                        'LowerArmFront': '-36.86349', 
                        'LowerArm':      '-0.4033567', 
                        'UpperLegFront': '3.002661', 
                        'UpperLeg':      '-0.003749454', 
                        'LowerBody':     '-0.05592484', 
                        'MiddleBody':    '0', 
                        'LowerLeg':      '-0.007670409', 
                        'Foot':          '0.9779164'
                    },
                    'ShouldStandUpright': true, 
                    'Animated': false,
                    'UprightForceMultiplier': 0.1,
                    'Rigidity': 5.2
                },

                'JT_Victory': {
                    'LimbAngles': {
                        'LowerArmFront': '-42.56367', 
                        'Head':          '0.002119066', 
                        'LowerArm':      '-56.61966', 
                        'LowerBody':     '-0.005534801', 
                        'LowerLeg':      '-0.0006021239', 
                        'LowerLegFront': '0.002093026', 
                        'FootFront':     '-1.31155', 
                        'UpperLeg':      '-0.002053326', 
                        'UpperLegFront': '-0.001093257', 
                        'Foot':          '-1.019251', 
                        'UpperArm':      '205.2887', 
                        'MiddleBody':    '0', 
                        'UpperArmFront': '-123.5136', 
                        'UpperBody':     '-0.004975366'
                    },
                    'ShouldStandUpright': true, 
                    'Animated': false,
                    'UprightForceMultiplier': 1.1,
                    'Rigidity': 5.2       
                },

                'JT_Club_1': {
                    'LimbAngles': {
                        'LowerLegFront':      '42.26661', 
                        'LowerArm':           '-44.49521', 
                        'Foot':               '-17.01438', 
                        'UpperArmFront':      '169.3435', 
                        'UpperArm':           '167.8358', 
                        'LowerArmFront':      '-43.21984', 
                        'FootFront':          '-13.52342', 
                        'LowerLeg':           '15.21689', 
                        'UpperLeg':           '5.276407', 
                        'MiddleBody':         '0', 
                        'UpperBody':          '0.1287038', 
                        'UpperLegFront':      '-23.86698', 
                        'LowerBody':          '-3.01087', 
                        'Head':               '5.615793'
                    },
                    'ShouldStandUpright': true, 
                    'Animated': false,
                    'UprightForceMultiplier': 1.1,
                    'Rigidity': 5.2       
                },

                'JT_Club_2': {   
                    'LimbAngles': {
                        'LowerArmFront': '-75.48338', 
                        'UpperBody':     '-0.003524378', 
                        'Head':          '0.005585814', 
                        'LowerArm':      '-77.10894', 
                        'UpperArm':      '-72.16587', 
                        'MiddleBody':    '0', 
                        'UpperArmFront': '-70.63633', 
                        'Foot':          '-17.0221', 
                        'LowerBody':     '-3.282435', 
                        'FootFront':     '-16.31904', 
                        'LowerLegFront': '32.53976', 
                        'LowerLeg':      '30.60237', 
                        'UpperLegFront': '-27.16502', 
                        'UpperLeg':      '-0.001768379'
                    },
                    'ShouldStandUpright': true, 
                    'Animated': false,
                    'UprightForceMultiplier': 1.1,
                    'Rigidity': 5.2       
                }}";

            Poses = JsonConvert.DeserializeObject<Dictionary<string, PoseSet>>(JTCustomPoses);

        }

    }
}
